import 'package:flutter/cupertino.dart';
import 'package:pt2_emulated_flutter/core/BaseResponse.dart';
import 'package:pt2_emulated_flutter/core/credit_card_validator.dart';
import 'package:pt2_emulated_flutter/payment_transaction/model/credit_card_model.dart';
import 'package:pt2_emulated_flutter/payment_transaction/model/merchant_info_entity.dart';
import 'package:pt2_emulated_flutter/payment_transaction/model/prepare_transaction_request.dart';
import 'package:pt2_emulated_flutter/payment_transaction/model/prepare_transaction_response_entity.dart';
import 'package:pt2_emulated_flutter/payment_transaction/model/required_data_model.dart';
import 'package:pt2_emulated_flutter/payment_transaction/repo/payment_transaction_repo.dart';

class PaymentTransactionProvider extends ChangeNotifier {
  PaymentTransactionRepo repo = new PaymentTransactionRepo();
  CreditCardValidator _validator = new CreditCardValidator();
  MerchantInfoEntity merchantInfoEntity;
  bool isLoading = false;
  String ccNumberValidationError;
  String ccHolderNameValidationError;
  String ccCvvValidationError;
  String ccExpiryDateValidationError;

  Future<bool> validateSecretAndGetMerchantInfo(RequiredDataModel model) async {
    bool isValid = false;
    try {
      await repo
          .validateSecret(model.merchantEmail, model.secretKey)
          .then((resp) => {isValid = resp.status});
      await getMerchantInfo(model).then((resp) => {isValid = resp});
    } catch (e) {
      print("logd validateSecret $e");
      isValid = false;
    }
    return isValid;
  }

  Future<bool> getMerchantInfo(RequiredDataModel model) async {
    bool isValid = false;
    try {
      await repo
          .getMerchantInfo(model)
          .then((resp) => {isValid = onMerchantInfoFinished(resp)});
    } catch (e) {
      print("logd getMerchantInfo  $e");
      isValid = false;
    }
    return isValid;
  }

  Future<PayTabsBaseResponse<PrepareTransactionResponseEntity>>
      prepareTransaction(CreditCardModel creditCardModel,
          RequiredDataModel requiredDataModel) async {
    try {
      setLoading(true);
      if (validateCC(creditCardModel)) {
        var payTabsBaseResponse =
            await repo.prepareTransaction(creditCardModel, requiredDataModel);
        setLoading(false);
        return payTabsBaseResponse;
      }else{
        setLoading(false);
        return PayTabsBaseResponse(
            businessCode: "",
            data: null,
            messages: "Please type valid data",
            status: false);
      }
    } catch (e) {
      setLoading(false);
      return PayTabsBaseResponse(
          businessCode: "",
          data: null,
          messages: "SomeThing went wrong",
          status: false);
    }
  }

  void setLoading(bool b) {
    isLoading = b;
    notifyListeners();
  }

  bool onMerchantInfoFinished(PayTabsBaseResponse<MerchantInfoEntity> resp) {
    if (resp.status) merchantInfoEntity = resp.data;
    return resp.status;
  }

  bool validateCC(CreditCardModel cc) {
    return validateHolderName(cc.cardHolderName) &
        validateCvv(cc.cvvCode) &
        validateExpiryDate(cc.expiryDate) &
        validateNumber(cc.cardNumber);
  }

  bool validateNumber(String number) {
    var result = _validator.validateCCNum(number);
    ccNumberValidationError = result.message;
    return result.isValid;
  }

  bool validateExpiryDate(String date) {
    var result = _validator.validateExpDate(date);
    ccExpiryDateValidationError = result.message;
    return result.isValid;
  }

  bool validateHolderName(String name) {
    var result = _validator.validateName(name);
    ccHolderNameValidationError = result.message;
    notifyListeners();
    return result.isValid;
  }

  bool validateCvv(String cvv) {
    var result = _validator.validateSecurityCode(cvv);
    ccCvvValidationError = result.message;
    return result.isValid;
  }
}
