import 'dart:convert';

import 'package:dio/dio.dart';
import 'package:http/http.dart';
import 'package:http_interceptor/http_client_with_interceptor.dart';
import 'package:pt2_emulated_flutter/core/BaseResponse.dart';
import 'package:pt2_emulated_flutter/core/http_interceptor.dart';
import 'package:pt2_emulated_flutter/core/logging_interceptor.dart';
import 'package:pt2_emulated_flutter/payment_transaction/model/credit_card_model.dart';
import 'package:pt2_emulated_flutter/payment_transaction/model/merchant_info_entity.dart';
import 'package:pt2_emulated_flutter/payment_transaction/model/prepare_transaction_request.dart';
import 'package:pt2_emulated_flutter/payment_transaction/model/prepare_transaction_response_entity.dart';
import 'package:pt2_emulated_flutter/payment_transaction/model/required_data_model.dart';
import 'package:pt2_emulated_flutter/payment_transaction/model/validate_secret_response_entity.dart';

import '../../constants.dart';

class PaymentTransactionRepo {
  Dio dio = new Dio();

  PaymentTransactionRepo() {
    dio.interceptors.add(ApiInterceptor());
  }

  Future<PayTabsBaseResponse<ValidateSecretResponseEntity>> validateSecret(
      merchantEmail, secretKey) async {
    try {
      var endpoint = "$BaseUrl/apiv3/validate_secret_key";
      FormData formData = new FormData.fromMap({
        "merchant_email": merchantEmail,
        "secret_key": secretKey,
      });
      var response = await dio.post<String>('$endpoint', data: formData);
      var responseEntity =
          ValidateSecretResponseEntity().fromJson(json.decode(response.data));
      return PayTabsBaseResponse<ValidateSecretResponseEntity>(
          businessCode: "",
          data: responseEntity,
          status: responseEntity.responseCode == SUCCESS_RESPONSE_CODE,
          messages: null);
    } catch (e) {
      print("logd validateSecret $e");
      return PayTabsBaseResponse<ValidateSecretResponseEntity>(
          businessCode: "", data: null, status: false, messages: null);
    }
  }

  Future<PayTabsBaseResponse<MerchantInfoEntity>> getMerchantInfo(
      RequiredDataModel model) async {
    try {
      var endpoint = "$BaseUrl/apiv3/get_merchant_info";
      FormData formData = new FormData.fromMap({
        "merchant_email": model.merchantEmail,
        "secret_key": model.secretKey,
        "get_logo": "jhamza@paytabs.com",
        "amount": model.amount,
        "currency_code": model.currency,
        "is_tokenization": model.isTokenizationEnabled,
        "is_preauth": model.isPreAuth,
        "is_existing_customer": model.isExistingCustomer,
        "pt_token": "jhamza@paytabs.com",
        "pt_customer_email": model.customerEmail,
        "pt_customer_password": "qwertyui",
      });
      var response = await dio.post<String>('$endpoint', data: formData);
      var responseEntity =
          MerchantInfoEntity().fromJson(json.decode(response.data));

      return PayTabsBaseResponse<MerchantInfoEntity>(
          businessCode: "",
          data: responseEntity,
          status: responseEntity.responseCode == SUCCESS_RESPONSE_CODE,
          messages: null);
    } catch (e) {
      print("logd getMerchantInfo $e");
      return PayTabsBaseResponse<MerchantInfoEntity>(
          businessCode: "", data: null, status: false, messages: null);
    }
  }

  Future<PayTabsBaseResponse<PrepareTransactionResponseEntity>>
      prepareTransaction(
          CreditCardModel ccModel, RequiredDataModel requiredDataModel) async {
    try {
      var endpoint = "$BaseUrl/apiv3/prepare_transaction";
      FormData formData = new FormData.fromMap({
        "merchant_email": requiredDataModel.merchantEmail,
        "secret_key": requiredDataModel.secretKey,
        "amount": requiredDataModel.amount,
        "title": requiredDataModel.transactionTitle,
        "cc_first_name": ccModel.getFirstName(),
        "cc_last_name": ccModel.getLastName(),
        "card_exp": ccModel.getExpiryDate(),
        "cvv": ccModel.cvvCode,
        "card_number": ccModel.cardNumber,
        "original_assignee_code": requiredDataModel.originalAssigneeCode,
        "currency": requiredDataModel.currency,
        "email": requiredDataModel.email,
        "skip_email": requiredDataModel.skipEmail,
        "phone_number": requiredDataModel.customerPhoneNumber,
        "order_id": requiredDataModel.orderId,
        "product_name": requiredDataModel.productName,
        "customer_email": requiredDataModel.customerEmail,
        "country_billing": requiredDataModel.countryBilling,
        "address_billing": requiredDataModel.addressBilling,
        "city_billing": requiredDataModel.cityBilling,
        "state_billing": requiredDataModel.stateBilling,
        "postal_code_billing": requiredDataModel.postalCodeBilling,
        "country_shipping": requiredDataModel.countryShipping,
        "address_shipping": requiredDataModel.addressShipping,
        "city_shipping": requiredDataModel.cityShipping,
        "state_shipping": requiredDataModel.stateShipping,
        "postal_code_shipping": requiredDataModel.postalCodeShipping,
        "exchange_rate": requiredDataModel.exchangeRate,
        "is_tokenization": requiredDataModel.isTokenizationEnabled,
        "is_existing_customer": requiredDataModel.isExistingCustomer
      });
      var response = await dio.post<String>('$endpoint', data: formData);
      var responseEntity = PrepareTransactionResponseEntity()
          .fromJson(json.decode(response.data));

      return PayTabsBaseResponse<PrepareTransactionResponseEntity>(
          businessCode: "",
          data: responseEntity,
          status: responseEntity.responseCode == SUCCESS_PREPARE_TRANSACTION_RESPONSE_CODE,
          messages: responseEntity.result);
    } catch (e) {
      return PayTabsBaseResponse<PrepareTransactionResponseEntity>(
          businessCode: "", data: null, status: false, messages: "SomeThing went wrong please try again");
    }
  }
}
