import 'dart:async';

import 'package:dynamic_theme/dynamic_theme.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';
import 'package:pt2_emulated_flutter/core/BaseResponse.dart';
import 'package:pt2_emulated_flutter/payment_transaction/model/credit_card_model.dart';
import 'package:pt2_emulated_flutter/payment_transaction/model/prepare_transaction_response_entity.dart';
import 'package:pt2_emulated_flutter/payment_transaction/model/required_data_model.dart';
import 'package:pt2_emulated_flutter/payment_transaction/provider/payment_transaction_provider.dart';
import 'package:pt2_emulated_flutter/payment_verification/widget/D3sWebView.dart';
import 'package:pt2_emulated_flutter/payment_verification/widget/VerifyPaymentWidget.dart';

import '../../constants.dart';
import 'credit_card_form.dart';

class PaymentPage extends StatefulWidget {
  final RequiredDataModel requiredMerchantData;

  PaymentPage({
    Key key,
    @required this.requiredMerchantData,
  }) : super(key: key);

  @override
  _PaymentPageState createState() => _PaymentPageState();
}

class _PaymentPageState extends State<PaymentPage> {
  var _scaffoldKey = GlobalKey<ScaffoldState>();
  Future validateInfo;
  CreditCardModel _creditCardModel;

  @override
  void initState() {
    // TODO: implement initState
    super.initState();
    validateInfo =
        Provider.of<PaymentTransactionProvider>(context, listen: false)
            .validateSecretAndGetMerchantInfo(widget.requiredMerchantData);
    _creditCardModel = new CreditCardModel("", "", "", "", true);
  }

  @override
  Widget build(BuildContext context) {
    var provider = Provider.of<PaymentTransactionProvider>(context);
    return MaterialApp(
      theme: widget.requiredMerchantData.themeData,
      home: Scaffold(
        key: _scaffoldKey,
        appBar: AppBar(
          title: Center(child: Text("Make a Payment")),
        ),
        body: FutureBuilder<bool>(
          future: validateInfo,
          builder: (context, snapshot) {
            if (snapshot.connectionState == ConnectionState.done) {
              if (snapshot.data)
                return buildCreditCardForm(provider);
              else
                return Center(
                  child: Text("Merchant validate secret Failed"),
                );
            } else {
              return Center(child: CircularProgressIndicator());
            }
          },
        ),
      ),
    );
  }

  buildCreditCardForm(PaymentTransactionProvider provider) {
    return SingleChildScrollView(
      child: Column(
        children: [
          CreditCardForm(
            onCreditCardModelChange: (creditCardModel) {
              _creditCardModel = creditCardModel;
            },
          ),
          Padding(
            padding: EdgeInsets.all(10),
            child: paymentButton(provider),
          ),
        ],
      ),
    );
  }

  paymentButton(PaymentTransactionProvider provider) {
    return RaisedButton(
      padding: EdgeInsets.fromLTRB(20.0, 15.0, 20.0, 15.0),
      elevation: 5,
      shape: OutlineInputBorder(
        borderRadius: BorderRadius.only(
            topLeft: Radius.circular(30),
            topRight: Radius.circular(30),
            bottomRight: Radius.circular(30),
            bottomLeft: Radius.circular(30)),
        borderSide: BorderSide(style: BorderStyle.none),
      ),
      onPressed: () {
        if (provider.isLoading == false)
          provider
              .prepareTransaction(_creditCardModel, widget.requiredMerchantData)
              .then((value) => {onTransactionPrepared(value)});
      },
      child: (provider.isLoading)
          ? CircularProgressIndicator()
          : Text("Pay now",
              textAlign: TextAlign.center,
              style: TextStyle(color: Colors.blue)),
    );
  }

  @override
  void dispose() {
    super.dispose();
  }

  void showSnackBar(errorStr) {
    _scaffoldKey.currentState.showSnackBar(SnackBar(
      content: Text(errorStr),
    ));
  }

  onTransactionPrepared(
      PayTabsBaseResponse<PrepareTransactionResponseEntity> resp) {
    if (resp.status &&
        resp.data.responseCode == SUCCESS_PREPARE_TRANSACTION_RESPONSE_CODE) {
      Navigator.push(
          context,
          MaterialPageRoute(
              builder: (context) => D3sWebView(
                    d3Url: resp.data.redirectUrl,
                    returnUrl: resp.data.returnUrl,
                  ))).then((value) => {onD3sFinished(value)});
    } else {
      showSnackBar(resp.messages);
    }
  }

  onD3sFinished(int value) {
    print("logd value  $value");
    if (value == 0) {
      showGeneralDialog(
          context: context,
          pageBuilder: (c, cc, ccc) {
            return VerifyPaymentWidget(
              merchantEmail: "",
              secretKey: "",
              transactionId: "",
            );
          }).then((value) => {onVerifyPaymentFinished(value)});
    }
  }

  onVerifyPaymentFinished(value) {}
}
